#pragma rtGlobals=3		// Use modern global access method and strict wave access.
#pragma version = 0.13
#pragma ModuleName=VoigtDeconvolve
#include "FWHM"
#requiredPackages "Herix;"


Menu "Herix"
	SubMenu "Simple Peak Shape Fits"
		SubMenu "Voigt Fits"
			MenuItemIfTopGraphHerixEscan("Fit 1 Convolved Voigt on Top Graph"),DoVoigtConvolvedFit($"",$"")
			MenuItemIfTopGraphHerixEscan("Fit Multi-Peak Convolved Voigts on Top Graph"),FitMultiVoigtSpectrum($"",$"",NaN,NaN)
			"Set Multiple Voigt Peaks Coefs", SetMultiVoigtPeakCoefs($"",$"",NaN,NaN)
		End
		SubMenu "Gaussian Fits"
			MenuItemIfTopGraphHerixEscan("Do Gaussian Convolved Fit on Top Graph"),DoGaussConvolvedFit($"",$"")
		End
	End
End



//  ================================================================================  //
//  ======================== Start Fit Multiple Voigt Peaks ========================  //

Function/WAVE FitMultiVoigtSpectrum(yw,xw,Npeaks,separation,[cw,useResolution,pLo,pHi,printIt])
	Wave yw,xw					// (x,y) data
	Variable Npeaks			// number of peaks to fit
	Variable separation		// min or expected separation between peaks
	Wave cw						// optional input wave to receive coeffs
	Variable useResolution	// convolve with measured resolution
	Variable pLo,pHi			// point range to fit
	Variable printIt			// forces printing
	if (ParamIsDefault(cw))
		Wave cw = $""
	endif
	useResolution = ParamIsDefault(useResolution) ? NaN : !(!useResolution)
	useResolution = numtype(useResolution) ? NaN : useResolution
	pLo = ParamIsDefault(pLo) ? -Inf : pLo
	pHi = ParamIsDefault(pHi) ? Inf : pHi
	printIt = ParamIsDefault(printIt) ? NaN : !(!printIt)
	printIt = numtype(printIt) ? strlen(GetRTStackInfo(2))<1 : printIt

	if (!(Npeaks>0) && WaveExists(cw))
		Npeaks = round((DimSize(cw,0)-1)/4)
	endif

	if (numtype(Npeaks+separation) || Npeaks<1 || separation<=0)
		Prompt Npeaks,"Number of Voigt Peaks to Fit", popup, "auto;1;2;3;4;5;6;7;8;9;10"
		Prompt separation,"Min Separation between peaks (in points not eV)"
		DoPrompt "Number of Peaks",separation,Npeaks
		if (V_flag)
			return $""
		endif
		Npeaks = Npeaks==1 ? NaN : Npeaks-1
	endif
	if (numtype(separation) || separation<2)
		printf "ERROR -- FitMultiVoigtSpectrum(), cannot have separation=%g\r",separation
		return $""
	endif

	String fitInfo = WhatToFitInfo("",pLo=pLo,pHi=pHi)
	String fldr=StringByKey("fldr",fitInfo,"=")
	if (!WaveExists(yw))
		Wave yw = $(fldr+StringByKey("yWave",fitInfo,"="))
	endif
	if (!WaveExists(xw))
		Wave xw = $(fldr+StringByKey("xWave",fitInfo,"="))
	endif
	pLo = NumberByKey("pLo",fitInfo,"=")
	pHi = NumberByKey("pHi",fitInfo,"=")
	Variable xLo=NumberByKey("xLo",fitInfo,"="), xHi=NumberByKey("xHi",fitInfo,"=")

	if (!WaveExists(yw) || numtype(pLo+pHi))
		print "ERROR -- FitMultiVoigtSpectrum(), Cannot find data waves from top graph"
		return $""
	elseif (xHi*xLo >= 0)
		print "ERROR -- FitMultiVoigtSpectrum(), selected range does not span origin"
		return $""
	endif
	String fldrSav0= GetDataFolder(1)

	if (numtype(useResolution))
		DoAlert 2,"Convolve with Measured Resolution?"
		if (V_flag==3)
			return $""
		endif
		useResolution = V_flag==1
	endif
	String units = SelectString(WaveExists(xw),WaveUnits(yw,0),WaveUnits(xw,-1))
	Variable useP = pLo>0 || pHi<DimSize(yw,0)-1

	if (printIt)
		String xname = SelectString(WaveExists(xw),"$\"\"",GetWavesDataFolder(xw,2))
		printf "FitMultiVoigtSpectrum(%s,%s, %g,%g",GetWavesDataFolder(yw,2),xname,Npeaks,separation
		if (!ParamIsDefault(cw))
			printf ", cw=%s",GetWavesDataFolder(cw,2)
		endif
		if (!ParamIsDefault(useResolution) || useResolution)
			printf ", useResolution=%g",useResolution
		endif
		if (useP)
			printf ", pLo=%g,pHi=%g",pLo,pHi
		endif
		printf ")\r"
	endif
	if (!WaveExists(cw))								// creates a first guess at the coefficients for a spectrum
		SetDataFolder fldr
		Wave cw = SetMultiVoigtPeakCoefs(yw,xw,Npeaks,separation,pLo=pLo,pHi=pHi)
		SetDataFolder fldrSav0
	endif
	if (!WaveExists(cw))
		print "ERROR -- FitMultiVoigtSpectrum(), Could not get the Coefficients wave"
		return $""
	endif
	Npeaks = (DimSize(cw,0)-1)/4						// number of coefs must 2*4 + 1
	if (Npeaks!=round(Npeaks))
		print "ERROR -- FitMultiVoigtSpectrum(), Could not get the Coefficients wave"
		return $""
	endif

	String startCW=vec2str(cw)
	if (printIt)
		printf "	Starting Fit with coefficients \"%s\" = %s\r",GetWavesDataFolder(cw,2),startCW
	endif

	Wave resolutionWave=root:resolutionWave		// ensure that the resolution wave exists
	if (!WaveExists(resolutionWave))
		Wave resolutionWave=herix#SetResolutionWave(yw)
	endif
	if (!WaveExists(resolutionWave))
		DoAlert 0, "Unable to get resolution wave"
		return $""
	endif

	Wave weight=$""
	if (StringMatch(WaveUnits(yw,-1),"counts"))
		MatrixOp/FREE weight = rec(sqrt(yw))		// for "counts", use sqrt(N) weighting, weight is 1/error
	endif
	String fitName=SelectString(useResolution,"VoigtN","VoigtNConvolved")
	FUNCREF VoigtN fitFunc = $fitName
	SetDataFolder fldr
	Variable i,err=0, V_FitError=0, V_FitQuitReason=0
	FuncFit/Q/NTHR=0 fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D/NWOK
	//	FuncFit/Q/NTHR=0/M=2 fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D/NWOK		// makes M_Covar
	err = V_FitError || V_FitQuitReason
	for (i=2;i<DimSize(cw,0);i+=4)
		cw[i] = abs(cw[i])								// shape and width are always positive
		cw[i+2] = abs(cw[i+2])
	endfor
	Wave W_sigma=W_sigma
	if (V_FitError || V_FitQuitReason)
		print "Fit Failed ************************"
		print StringFromList(V_FitError,"","Any error;Singular matrix;Out of memory;Function returned NaN or INF;Fit function requested stop;Reentrant curve fitting;")
		if (V_FitQuitReason)
			print StringFromList(V_FitQuitReason,"","the iteration limit was reached;the user stopped the fit;the limit of passes without decreasing chi-square was reached;")
		endif
	endif
	SetDataFolder fldrSav0

	if (printIt && !err)
		Variable amp,hwG,hwL,hwV, dAmp,dhwG,dhwL,dhwV
		printf "	Base	= %s,   (chi^2 = %g)\r",ValErrStr(cw[0],W_sigma[0],sp=1),roundSignificant(V_chisq,5)
		for (i=0;i<Npeaks;i+=1)
			Amp = cw[4*i+1]*sqrt(Pi)/cw[4*i+2]
			dAmp = (sqrt(2)*W_sigma[4*i+1]/cw[4*i+1]) * Amp
			hwG = sqrt(ln(2))/cw[4*i+2]
			dhwG = W_sigma[4*i+2]/cw[4*i+2] * hwG
			hwL = cw[4*i+4]/cw[4*i+2]
			dhwL = sqrt((W_sigma[4*i+4]/cw[4*i+4])^2 + (W_sigma[4*i+4]/cw[4*i+4])^2) * hwL
			hwV = hwL/2 + sqrt(0.25*(hwL^2) + hwG^2)
			dhwV = sqrt(2*(dhwL/hwL)^2 + (dhwG/hwG)^2) * hwV
			printf "	peak %g:,   Amp=%s,   FWHM Gauss=%s,   FWHM Lorentz=%s,   FWHM Voigt=%s\r",i,ValErrStr(Amp,dAmp,sp=1),ValErrStr(2*hwG,2*dhwG,sp=1),ValErrStr(2*hwL,2*dhwL,sp=1),ValErrStr(2*hwV,2*dhwV,sp=1)
		endfor
		printf "	%s = %s\r",NameOfWave(cw),vec2str(cw)
		printf "	%s = %s\r",NameOfWave(W_sigma),vec2str(W_sigma)
	endif

	String wnote="waveClass=VoigtNCoefs;"
	wnote = ReplaceStringByKey("sourceY",wnote,GetWavesDataFolder(yw,2),"=")
	if (WaveExists(xw))
		wnote = ReplaceStringByKey("sourceX",wnote,GetWavesDataFolder(xw,2),"=")
	endif
	if (useP)
		String str
		sprintf str,"[%g,%g]",pLo,pHi
		wnote = ReplaceStringByKey("subrange",wnote,str,"=")
	endif
	wnote = ReplaceStringByKey("function",wnote,fitName,"=")
	wnote = ReplaceStringByKey("startingCoefs",wnote,startCW,"=")
	if (WaveExists(resolutionWave))
		wnote = ReplaceStringByKey("resolutionWave",wnote,GetWavesDataFolder(resolutionWave,2),"=")
	endif
	if (!err)
		wnote = ReplaceNumberByKey("chisq",wnote,roundSignificant(V_chisq,5),"=")
		wnote = ReplaceStringByKey("W_sigma",wnote,vec2str(W_sigma),"=")
	endif
	Note/K cw, wnote
	return cw
End



Function/WAVE SetMultiVoigtPeakCoefs(yw,xw,Npeaks,separation,[cw,pLo,pHi,printIt])		// creates a first guess at the coefficients for a spectrum
	Wave yw,xw				// (x,y) data
	Variable Npeaks		// number of peaks to fit
	Variable separation	// min or expected separation between peaks
	Wave cw					// optional input wave to receive coeffs
	Variable pLo,pHi		// point range to fit
	Variable printIt		// forces printing
	if (ParamIsDefault(cw))
		Wave cw = $""
	endif
	pLo = ParamIsDefault(pLo) ? -Inf : pLo
	pHi = ParamIsDefault(pHi) ? Inf : pHi
	printIt = ParamIsDefault(printIt) ? NaN : !(!printIt)
	printIt = numtype(printIt) ? strlen(GetRTStackInfo(2))<1 : printIt

	String fitInfo = WhatToFitInfo("",pLo=pLo,pHi=pHi)
	String fldr=StringByKey("fldr",fitInfo,"=")
	if (!WaveExists(yw))
		Wave yw = $(fldr+StringByKey("yWave",fitInfo,"="))
	endif
	if (!WaveExists(xw))
		Wave xw = $(fldr+StringByKey("xWave",fitInfo,"="))
	endif
	pLo = NumberByKey("pLo",fitInfo,"=")
	pHi = NumberByKey("pHi",fitInfo,"=")
	if (!WaveExists(yw) || numtype(pLo+pHi))
		print "ERROR -- SetMultiVoigtPeakCoefs(), Cannot find data waves from top graph"
		return $""
	endif

	Variable sepTrial = separation>0 ? separation : DimSize(yw,0)/3
	String list = FindAllPeaks(yw,sepTrial,"",-Inf)	// returns approximate initial peak positions
	if (numtype(Npeaks+separation) || Npeaks<1 || separation<=2)
		Variable listLen = ItemsInList(list)
		Npeaks = numtype(Npeaks) || Npeaks<1 ? listLen : Npeaks
		separation = numtype(separation) || separation<2 ? abs(str2num(StringFromList(1,list))-str2num(StringFromList(0,list))) : separation
		separation = numtype(separation) || separation<2 ? 30 : separation
		Prompt Npeaks,"Number of Voigt Peaks to Fit", popup, expandRange("1-"+num2istr(listLen),";")
		Prompt separation,"Min Separation between peaks (in points not eV)"
		DoPrompt "Number of Peaks",separation,Npeaks
		if (V_flag)
			return $""
		endif
	endif
	if (numtype(Npeaks+separation) || Npeaks<1 || separation<=0)
		printf "ERROR -- SetMultiVoigtPeakCoefs(), cannot have Npeaks=%g, or separation=%g\r",Npeaks,separation
		return $""
	endif
	Variable Ncw = 4*Npeaks + 1

	if (!WaveExists(cw))
		Make/N=(Ncw)/O/D Vcoefs
		Wave cw = Vcoefs
	else
		Redimension/N=(Ncw)/D cw
	endif
	SetDimLabel 0,0,offset,cw
	Variable i
	for (i=0;i<Npeaks;i+=1)
		SetDimLabel 0,4*i+1,amplitude,cw
		SetDimLabel 0,4*i+2,width,cw
		SetDimLabel 0,4*i+3,center,cw
		SetDimLabel 0,4*i+4,shape,cw
	endfor

	MatrixOp/FREE offset0 = minVal(abs(yw))
	cw[0] = offset0[0]
	cw[0] = cw[0]>0 ? cw[0] : min(abs(WaveMin(yw)),abs(WaveMax(yw)))/20

	Variable ipnt, xVal
	for (i=0;i<Npeaks;i+=1)
		ipnt = str2num(StringFromList(i,list))
		xVal = WaveExists(xw) ? xw[ipnt] : pnt2x(yw,ipnt)
		cw[4*i+1] = yw[ipnt]			// amplitude
		cw[4*i+2] = 16/separation		// width
		cw[4*i+3] = xVal					// center
		cw[4*i+4] = 0.5					// shape
	endfor

	if (printIt)
		printf "Created coefficients wave  \"%s\" = %s\r",NameOfWave(cw),vec2str(cw)
	endif
	return cw
End



Function VoigtNConvolved(wc,dx) : FitFunc	// This is the fitting function N Voigts (4*N+1 coefficients)
	WAVE wc			// fitting coefficients (things to determine)
	Variable dx		// x value
	//	wc[0]		offset

	// peak 1
	//	wc[1]		amplitude_1
	//	wc[2]		width_1 (for shape=0, wc[2] = 1/sigma/sqrt(2)
	//	wc[3]		center_1
	//	wc[4]		shape_1, 0=Gaussian, inf=Lorentzian, 1=50/50

	// peak 2
	//	wc[5]		amplitude_2
	//	wc[6]		width_2 (for shape=0, wc[2] = 1/sigma/sqrt(2)
	//	wc[7]		center_2
	//	wc[8]		shape_2, 0=Gaussian, inf=Lorentzian, 1=50/50

	// for more peaks...

	Variable Np=(DimSize(wc,0)-1)/4					// number of coefs must be 4*Np + 1
	if (Np!=round(Np))
		return NaN
	endif

	Wave resolutionWave= root:resolutionWave	// measured resolution
	Duplicate/FREE resolutionWave, Vsw
	Duplicate/FREE wc, vw
	Variable i
	for (i=2;i<DimSize(wc,0);i+=4)
		vw[i] = abs(vw[i])								// widths
		vw[i+2] = abs(vw[i+2])							// shapes
	endfor
	vw[0] = 0												// add in baseline after convolution
	Vsw = resolutionWave(x) * VoigtFit(vw,dx+x)
	//	amplitude		= coef[1]*sqrt(pi)/coef[2]
	//	HWHM Gauss		= sqrt(ln(2))/coef[2]
	//	HWHM Lorentz	= coef[4]/coef[2] 
	//	HWHM Voigt		= wl/2 + sqrt( wl^2/4 + wg^2)

	Variable result = sum(Vsw) + wc[0]
	return result
End

Function VoigtN(wc,dx) : FitFunc	// This is the fitting function N Voigts (4*N+1 coefficients)
	WAVE wc			// fitting coefficients (things to determine)
	Variable dx		// x value
	//	wc[0]		offset

	// peak 1
	//	wc[1]		amplitude_1
	//	wc[2]		width_1 (for shape=0, wc[2] = 1/sigma/sqrt(2)
	//	wc[3]		center_1
	//	wc[4]		shape_1, 0=Gaussian, inf=Lorentzian, 1=50/50

	// peak 2
	//	wc[5]		amplitude_2
	//	wc[6]		width_2 (for shape=0, wc[2] = 1/sigma/sqrt(2)
	//	wc[7]		center_2
	//	wc[8]		shape_2, 0=Gaussian, inf=Lorentzian, 1=50/50

	// for more peaks...

	Variable Np=(DimSize(wc,0)-1)/4					// number of coefs must be 4*Np + 1
	if (Np!=round(Np))
		return NaN
	endif

	Duplicate/FREE wc, vw
	Variable i
	for (i=2;i<DimSize(wc,0);i+=4)
		vw[i] = abs(vw[i])
		vw[i+2] = abs(vw[i+2])
	endfor
	Variable result = VoigtFit(vw,dx)
	//	amplitude		= coef[1]*sqrt(pi)/coef[2]
	//	HWHM Gauss		= sqrt(ln(2))/coef[2]
	//	HWHM Lorentz	= coef[4]/coef[2] 
	//	HWHM Voigt		= wl/2 + sqrt( wl^2/4 + wg^2)

	return result
End

//  ========================= End Fit Multiple Voigt Peaks =========================  //
//  ================================================================================  //



//  ================================================================================  //
//  ========================= Start Fit Single Voigt Peak ==========================  //

//	DoVoigtConvolvedFit(root:spec458:HerixE,root:spec458:Ana5)
//   or
//	DoVoigtConvolvedFit(HerixE,Ana5)
//   or just
//	DoVoigtConvolvedFit($"", $"")
//
Function DoVoigtConvolvedFit(yData,xData,[slope,printIt])
	Wave yData,xData				// measured x,y data
	Variable slope					// flag, if true, include a sloped background, if false a flat background
	Variable printIt
	slope = ParamIsDefault(slope) ? 0 : slope
	slope = numtype(slope) ? 0 : !(!slope)
	printIt = ParamIsDefault(printIt) ? 0 : printIt
	printIt = numtype(printIt) ? 0 : !(!printIt)
	printIt = ParamIsDefault(printIt) ? !ParamIsDefault(slope) : printIt

	String gName=""				// using top graph
	String fitInfo = WhatToFitInfo(gName)
	Variable pA=NumberByKey("pLo",fitInfo,"="), pB=NumberByKey("pHi",fitInfo,"=")

	String fldr=StringByKey("fldr",fitInfo,"=")
	if (!WaveExists(yData))
		Wave yData = $(fldr+StringByKey("yWave",fitInfo,"="))
	endif
	if (!WaveExists(xData))
		Wave xData = $(fldr+StringByKey("xWave",fitInfo,"="))
	endif
	Variable useCursors = NumberByKey("useCursors",fitInfo,"=")	// got info from cursors
	Variable useXwave = WaveExists(xData)

	if (!WaveExists(yData))
		DoAlert 0,"Y data wave does not exist"
		return NaN
	elseif (WhichListItem(NameOfWave(yData),TraceNameList(gName,";",1))<0)
		DoAlert 0,"The data wave "+GetWavesDataFolder(yData,2)+" is not on the top graph"
		return NaN
	endif

	if (printIt)
		String str = "$\"\""
		if (useXwave)
			str = GetWavesDataFolder(xData,2)
		endif
		printf "DoVoigtConvolvedFit(%s, %s",GetWavesDataFolder(yData,2),str
		if (!ParamIsDefault(slope) || slope!=0)
			printf ", slope=%g", slope
		endif
		String pRange=""
		if (useCursors)
			sprintf pRange, "\t\tpoint range [%g, %g]",pA,pB
		endif
		printf ")%s\r",pRange
	endif

	Wave resolutionWave=root:resolutionWave	// ensure that the resolution wave exists
	if (!WaveExists(resolutionWave))
		Wave resolutionWave=herix#SetResolutionWave(yData)
	endif
	if (!WaveExists(resolutionWave))
		DoAlert 0, "Unable to get resolution wave"
		return NaN
	endif
	String units = WaveUnits(yData,0)
	if (useXwave)
		units = WaveUnits(xData,-1)
	endif

	if (useXwave)
		CurveFit/Q gauss yData[pA,pB] /X=xData /D
	else
		CurveFit/Q gauss yData[pA,pB] /D
	endif
	Wave W_coef=W_coef, W_sigma=W_sigma
	Variable fw = 2*W_coef[3]*sqrt(ln(2))
	Variable dfw = 2*sqrt(ln(2))*W_sigma[3]
	Variable gArea = GaussianIntegral(W_coef)
	printf "using a starting point from Gaussian fit = %s,   initial Gaussian FWHM = %g%.2g %s,  (chiSqr = %g)\r",vec2str(W_coef),fw,dfw,units,V_chisq

	Variable Nvw = slope ? 6 : 5
	Make/N=(Nvw)/D/FREE vw							// starting point for Voigt fit, these are not exact, just good starting point
	vw[0] = W_coef[0]									// offset
	vw[1] = gArea/W_coef[3]/sqrt(Pi)			// amplitude
	vw[2] = 1/W_coef[3]								// width parameter
	vw[3] = W_coef[2]									// center
	vw[4] = 0.2											// shape, 0=Gaussian, inf=Lorentzian, 1=50/50
	if (slope)
		vw[5] = 0.001
	endif
	if (useXwave)
		FuncFit/Q VoigtConvolved vw yData[pA,pB] /X=xData /D 
	else
		FuncFit/Q VoigtConvolved vw yData[pA,pB] /D 
	endif
	vw[4] = abs(vw[4])

	Variable fwG = 2*sqrt(ln(2))/vw[2]						// FWHM Gaussian part
	Variable fwL = 2*vw[4]/vw[2] 							// FWHM Lorentzian part
	Variable fwV= fwL/2 + sqrt( fwL^2/4 + fwG^2)		// FWHM Voigt
	Variable dfwV = NaN
	printf "DeConvolved Voigt parameters are %s,   (chiSqr = %g)\r",vec2str(vw),V_chisq
	printf "     (FWHM Gaussian = %.3g * FWHM Lorentzian = %.3g) (Voigt shape = %.3g),  FWHM Voigt = %.3g%.2g %s\r",fwG,fwL,vw[4],fwV,dfwV,units
	return fwV
End
//
Function VoigtConvolved(wc,dx) : FitFunc	// This is the fitting function
	WAVE wc			// fitting coefficients (things to determine)
	Variable dx		// x value
	//	wc[0]		offset
	//	wc[1]		amplitude
	//	wc[2]		width (for shape=0, wc[2] = 1/sigma/sqrt(2)
	//	wc[3]		center
	//	wc[4]		shape, 0=Gaussian, inf=Lorentzian, 1=50/50
	// wc[5]		optional slope of background
	Variable slope = (numpnts(wc)==6)

	Wave resolutionWave= root:resolutionWave	// measured resolution
	Duplicate/FREE resolutionWave, Vsw
	Duplicate/FREE wc, vw
	vw[0] = 0
	vw[4] = abs(vw[4])
	Vsw = resolutionWave(x) * VoigtFit(vw,x+dx)
	Variable result = sum(Vsw)

	if (slope)
		result += wc[0] + dx*wc[5]					// a sloped background
	else
		result += wc[0]									// a flat background
	endif
	return result
End


#if (IgorVersion() >= 7)
//	VoigtFunc(X, Y)
//	Computes the Voigt function using an approximation giving an accuracy better than one part in 10^5 over large range of input parameters.
//	VoigtFunc returns values from a normalized Voigt peak centered at X=0 for the given value of X. The X input is a normalized distance from the peak center:
//	X = sqrt(ln(2)) * (v-vo)/gammaG
//	where gammG is the Gaussian component half-width, and (v-vo) is the distance from the peak center.
//	The parameter Y is the shape parameter. When Y is zero, the peak is pure Gaussian. When Y approaches infinity, the shape is pure Lorentzian. When Y is one, the mix is half-and-half.
//	VoigtFunc was added in Igor Pro 7.00.
//	The VoigtFunc function returns values from a normalized peak that can be used as the basis for user-defined fitting functions. A typical fitting function might look like this:
//
Function VoigtFit(w,xx) : FitFunc				// the Igor 7 version of what was in the Igor 6 MultiPeakFit.xop
	Wave w
	Variable xx

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(xx) = Y0+Amp*VoigtFunc(width*(xx-x0),shape)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ xx
	//CurveFitDialog/ Coefficients 5
	//CurveFitDialog/ w[0] = Y0
	//CurveFitDialog/ w[1] = Amp
	//CurveFitDialog/ w[2] = width
	//CurveFitDialog/ w[3] = x0
	//CurveFitDialog/ w[4] = shape

	Variable i,i0, Np=floor( (DimSize(w,0)-1) / 4 )
	Variable total = w[0]
	for (i=0;i<Np;i+=1)
		i0 = 4*i + 1
		total += w[1+i0]*VoigtFunc(w[2+i0]*(xx-w[3+i0]),w[4+i0])
	endfor
	return total
End
//
//	Parameter w[0] sets the vertical offset, w[1] sets the amplitude, w[2] affects the width, w[3] sets 
//	the location of the peak and w[4] adjusts the shape (but also affects the amplitude). Note that w[2] 
//	cannot be taken directly to be width, as the shape parameter also affects the width.
//
//	After the fit, you can use the returned coefficients to calculate the area (a) along with the 
//	half width at half max for the Gaussian (wg), Lorentzian (wl) and the Voigt (wv). 
//	Assuming the coefficient wave is named w_coef:
//	a = w_coef[1]*sqrt(pi)/w_coef[2]
//	wg = sqrt(ln(2))/w_coef[2]
//	wl = w_coef[4]/w_coef[2] 
//	wv = wl/2 + sqrt( wl^2/4 + wg^2)
//	References
//	The approximation is described here:
//	Wells, R.J., Rapid Approximation to the Voigt/Faddeeva Function and Its Derivatives, 
//	Journal of Quantitative Sprectroscopy and Radiative Transfer, 1999.
#endif

//  ========================== End Fit Single Voigt Peak ===========================  //
//  ================================================================================  //



//  ================================================================================  //
//  ======================== Start Fit Single Gaussian Peak ========================  //

//	DoGaussConvolvedFit(root:spec458:HerixE,root:spec458:Ana5)
//   or
//	DoGaussConvolvedFit(HerixE,Ana5)
//
Function DoGaussConvolvedFit(xData,yData)
	Wave xData,yData				// measured x,y data

	if (!WaveExists(xData) || !WaveExists(yData))
		DoAlert 0,"X or Y data waves do not exist"
		return NaN
	elseif (WhichListItem(NameOfWave(yData),TraceNameList("",";",1))<0)
		DoAlert 0,"The data wave "+GetWavesDataFolder(yData,2)+" is not on the top graph"
		return NaN
	endif
	if (strlen(CsrInfo(A))==0 || strlen(CsrInfo(B))==0)
		DoAlert 0, "Could not find cursors A and B on the top graph"
		return NaN
	endif

	Wave resolutionWave=root:resolutionWave		// ensure that the resolution wave exists
	if (!WaveExists(resolutionWave))
		Wave resolutionWave=herix#SetResolutionWave(yData)
	endif
	if (!WaveExists(resolutionWave))
		DoAlert 0, "Unable to get resolution wave"
		return NaN
	endif

	CurveFit/Q gauss yData[pcsr(A),pcsr(B)] /X=xData /D
	Wave W_coef=W_coef, W_sigma=W_sigma
	Variable fw = 2*W_coef[3]*sqrt(ln(2))
	Variable dfw = 2*sqrt(ln(2))*W_sigma[3]
	printf "using a starting point from Gaussian fit = %s,   initial FWHM = %g%.2g,  (chiSqr = %g)\r",vec2str(W_coef),fw,dfw,V_chisq

	FuncFit/Q GaussConvolved W_coef yData[pcsr(A),pcsr(B)] /X=xData /D 
	fw = 2*W_coef[3]*sqrt(ln(2))
	dfw = 2*sqrt(ln(2))*W_sigma[3]
	printf "DeConvolved Gaussian parameters are %s,   final FWHM = %g%.2g,  (chiSqr = %g)\r",vec2str(W_coef),fw,dfw,V_chisq
	return fw
End
//
Function GaussConvolved(wc,dx) : FitFunc		// This is the fitting function
	WAVE wc			// fitting coefficients (things to determine)
	Variable dx		// x value
	//	gauss	Gaussian peak: y = K0+K1*exp(-((dx-K2)/K3)^2).

	Wave resolutionWave= root:resolutionWave	// measured resolution
	Duplicate/FREE resolutionWave, Ls
	Ls = resolutionWave(x) * wc[1]*exp(-((x+dx-wc[2])/wc[3])^2)
	return sum(Ls) + wc[0]
End

//  ========================= End Fit Single Gaussian Peak =========================  //
//  ================================================================================  //



//Static Function/C GetPointRange(gName)
//	String gName			// graph name, "" means top graph
//
//	if (strlen(WinList("*",";","WIN:1"))<1)
//		return cmplx(NaN,NaN)			// no graphs, so no answer
//	endif
//
//	String infoA=CsrInfo(A,gName), infoB=CsrInfo(B,gName)
//	Variable pLo=NumberByKey("POINT",infoA), pHi=NumberByKey("POINT",infoB)
////	if (strlen(infoA)==0 || strlen(infoB)==0 || numtype(pLo+pHi))
////		DoAlert 0, "Could not find cursors A and B on the top graph"
////		return NaN
////	endif
//	if (pLo>pHi)
//		Variable swap=pLo
//		pLo = pHi	
//		pHi = swap
//	endif
//
//	return cmplx(pLo,pHi)
//End
Static Function/T WhatToFitInfo(gName,[pLo,pHi])
	String gName			// graph name, "" means top graph
	Variable pLo,pHi		// point range to fit (optional)
	pLo = ParamIsDefault(pLo) ? -Inf : pLo
	pHi = ParamIsDefault(pHi) ? Inf : pHi

	String glist = WinList("*",";","WIN:1")	// list of all graph windows
	if (strlen(gName)<1)
		gname = StringFromList(0,glist)
	endif
	if (WhichListItem(gName,glist)<0 || strlen(gname)<1)
		return ""										// graph does not exist
	endif

	String infoA=CsrInfo(A,gName), infoB=CsrInfo(B,gName)
	String tname = StringByKey("TNAME",infoA)
	if (strlen(tname)<1)
		tname = StringFromList(0,TraceNameList(gname,";",1))
	endif
	Wave wy = TraceNameToWaveRef(gname,tname)
	Variable iyLast = DimSize(wy,0)-1
	if (!WaveExists(wy))
		return ""
	endif
	Wave wx = XWaveRefFromTrace(gname,tname)// xwave if one exists

	pLo = numtype(pLo) || pLo<0 ? NumberByKey("POINT",infoA) : limit(pLo,0,iyLast)
	pLo = numtype(pLo) ? 0 : pLo
	pHi = numtype(pHi) || pHi<0 ? NumberByKey("POINT",infoB) : limit(pHi,0,iyLast)
	pHi = numtype(pHi) ? iyLast : pHi
	if (pLo>pHi)
		Variable swap=pLo
		pLo = pHi	
		pHi = swap
	endif

	Variable xLo,xHi
	if (WaveExists(wx))
		xLo = wx[pLo]
		xHi = wx[pHi]
	else
		xLo = pnt2x(wx,pLo)
		xHi = pnt2x(wy,pHi)
	endif

	String list = ReplaceStringByKey("fldr","",GetWavesDataFolder(wy,1),"=")
	list = ReplaceStringByKey("yWave",list,NameOfWave(wy),"=")
	if (WaveExists(wx))
		list = ReplaceStringByKey("xWave",list,NameOfWave(wx),"=")
	endif
	list = ReplaceNumberByKey("pLo",list,pLo,"=")
	list = ReplaceNumberByKey("pHi",list,pHi,"=")
	list = ReplaceNumberByKey("xLo",list,xLo,"=")
	list = ReplaceNumberByKey("xHi",list,xHi,"=")
	list = ReplaceNumberByKey("useCursors",list,strlen(infoA)>1,"=")
	return list
End

Static Function/T FindAllPeaks(yw,separation,list,minVal)
	Wave yw
	Variable separation	// min or expected separation between peaks
	String list				// use "" on input
	Variable minVal		// use -Inf on input

	if (!WaveExists(yw) || !(separation>0))
		return ""
	endif
	Variable N=DimSize(yw,0)
	if (N<(separation) || WaveMax(yw)<minVal)
		return ""
	endif

	FindPeak/Q/P/B=(separation/3) yw
	if (V_flag)
		return list
	endif
	minVal = numtype(minVal) ? WaveMax(yw)/4 : minVal

	Variable peakVal0=NumberByKey("offset",note(yw))
	peakVal0 = numtype(peakVal0) ? 0 : peakVal0
	Variable PeakLoc = peakVal0+V_PeakLoc
	Variable lo = limit(round(V_PeakLoc-separation/2), 0,DimSize(yw,0)-1)
	Variable hi = limit(round(V_PeakLoc+separation/2), 0,DimSize(yw,0)-1)
	Make/N=(lo+1)/FREE yLo = yw[p]
	Make/N=(N-hi)/FREE yHi = yw[p+hi]
	Note/K yLo, ReplaceNumberByKey("offset","",peakVal0)
	Note/K yHi, ReplaceNumberByKey("offset","",peakVal0+hi)
//	printf "found peak at %g (using peakVal0=%g,  length = %g,  [%g, %g])\r",PeakLoc,peakVal0,N,peakVal0,peakVal0+N-1
//	printf "		V_PeakLoc =%g,  PeakLoc=%g,  V_PeakVal=%g (minVal=%g)  next try: [%g, %g]  and  [%g, %g]\r",V_PeakLoc,PeakLoc,V_PeakVal,minVal,peakVal0,lo+peakVal0, peakVal0+hi,peakVal0+N-1

	list += FindAllPeaks(yLo,separation,list,minVal) + FindAllPeaks(yHi,separation,list,minVal)
	if (V_PeakVal>minVal)
		//	list += num2istr(PeakLoc)+":"+num2str(V_PeakVal)+";" 
		list += num2istr(PeakLoc)+";" 
	endif

	return list
End
